# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by mktables.PL from e.g. Unicode.300.
# Any changes made here will be lost!
return <<'END';
00aa	
00b2	00b3
00b9	00ba
02b0	02b8
02e0	02e4
2070	
2074	207f
2120	
2122	
3192	319f
END
