# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by mktables.PL from e.g. Unicode.300.
# Any changes made here will be lost!
return <<'END';
1100		G
1101		GG
1102		N
1103		D
1104		DD
1105		R
1106		M
1107		B
1108		BB
1109		S
110a		SS
110b		
110c		J
110d		JJ
110e		C
110f		K
1110		T
1111		P
1112		H
1161		A
1162		AE
1163		YA
1164		YAE
1165		EO
1166		E
1167		YEO
1168		YE
1169		O
116a		WA
116b		WAE
116c		OE
116d		YO
116e		U
116f		WEO
1170		WE
1171		WI
1172		YU
1173		EU
1174		YI
1175		I
11a8		G
11a9		GG
11aa		GS
11ab		N
11ac		NJ
11ad		NH
11ae		D
11af		L
11b0		LG
11b1		LM
11b2		LB
11b3		LS
11b4		LT
11b5		LP
11b6		LH
11b7		M
11b8		B
11b9		BS
11ba		S
11bb		SS
11bc		NG
11bd		J
11be		C
11bf		K
11c0		T
11c1		P
11c2		H
END
