# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by mktables.PL from e.g. Unicode.300.
# Any changes made here will be lost!
return <<'END';
002c	
002e	
003a	
00a0	
060c	
fe50	
fe52	
fe55	
ff0c	
ff0e	
ff1a	
END
