# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by mktables.PL from e.g. Unicode.300.
# Any changes made here will be lost!
return <<'END';
fb55	
fb59	
fb5d	
fb61	
fb65	
fb69	
fb6d	
fb71	
fb75	
fb79	
fb7d	
fb81	
fb91	
fb95	
fb99	
fb9d	
fba3	
fba9	
fbad	
fbd6	
fbe7	
fbe9	
fbff	
fcdf	fcf4
fd34	fd3b
fe71	
fe77	
fe79	
fe7b	
fe7d	
fe7f	
fe8c	
fe92	
fe98	
fe9c	
fea0	
fea4	
fea8	
feb4	
feb8	
febc	
fec0	
fec4	
fec8	
fecc	
fed0	
fed4	
fed8	
fedc	
fee0	
fee4	
fee8	
feec	
fef4	
END
