# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by mktables.PL from e.g. Unicode.300.
# Any changes made here will be lost!
return <<'END';
01c5	
01c8	
01cb	
01f2	
1f88	1f8f
1f98	1f9f
1fa8	1faf
1fbc	
1fcc	
1ffc	
END
