# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by mktables.PL from e.g. Unicode.300.
# Any changes made here will be lost!
return <<'END';
0023	0025
002b	
002d	
00a2	00a5
00b0	00b1
066a	
09f2	09f3
0e3f	
17db	
2030	2034
207a	207b
208a	208b
20a0	20af
212e	
2212	2213
fb29	
fe5f	
fe62	fe63
fe69	fe6a
ff03	ff05
ff0b	
ff0d	
ffe0	ffe1
ffe5	ffe6
END
