# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by mktables.PL from e.g. Unicode.300.
# Any changes made here will be lost!
return <<'END';
ff61	ffbe
ffc2	ffc7
ffca	ffcf
ffd2	ffd7
ffda	ffdc
ffe8	ffee
END
