# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by mktables.PL from e.g. Unicode.300.
# Any changes made here will be lost!
return <<'END';
2460	2473
24b6	24ea
3260	327b
3280	32b0
32d0	32fe
END
