# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by mktables.PL from e.g. Unicode.300.
# Any changes made here will be lost!
return <<'END';
2102	
210a	2113
2115	
2119	211d
2124	
2128	
212c	212d
212f	2131
2133	2134
2139	
fb20	fb29
END
